var toSort = true;
var graphType;
const GT_ID = 'id';
const GT_MIC = 'mic';
var idArray;
var micArray;

function insertBlank(rows, x_base) {

    var newRows = new Array();

    for(var i = 0 ; i < rows.length ; i++) {
        newRows.push(rows[i]);
        if(rows[i].x == x_base) {
            if(rows.length - 1 != i) {
                newRows.push(new Object({id: "", mic: undefined, x: undefined, y: undefined, z: undefined}));
            }
        }
    }

    return newRows;
}

function collapseArray(multiArray) {

    var newArray = new Array();

    for(var key in multiArray) {
        newArray = newArray.concat(multiArray[key]);
    }

    return newArray;
}

function populateOptions(array) {
    $("option").remove();
    var elementForSelect = $("select#selectTF");
    var defaultValueSelect = document.createElement("option");
    defaultValueSelect.textContent = "Please select";
    elementForSelect.append(defaultValueSelect);
    for(var i = 0; i < array.length; i++) {
        var optionSelect = document.createElement("option");
        optionSelect.setAttribute("value", array[i]);
        optionSelect.text = array[i];
        elementForSelect.append(optionSelect);
    }

    var elementForFilter = $("select#filterTF");
    var defaultValueFilter = document.createElement("option");
    defaultValueFilter.textContent = "Please select";
    elementForFilter.append(defaultValueFilter);
    for(var j = 0; j < array.length; j++) {
        var optionFilter = document.createElement("option");
        optionFilter.setAttribute("value", array[j]);
        optionFilter.text = array[j];
        elementForFilter.append(optionFilter);
    }
}

function getIdArray() {
    Plotly.d3.csv('data.csv', function(err, rows) {
        var sortedArray = _.sortBy(rows, 'id');
        sortedArray = _.groupBy(sortedArray, function(n) {return n.id.trim();});
        var idArray_  = _.allKeys(sortedArray);
        idArray =idArray_;
    });
}

function getMicArray() {
    Plotly.d3.csv('data.csv', function(err, rows) {
        var sortedArray = _.sortBy(rows, 'mic');
        sortedArray = _.groupBy(sortedArray, function(n) {return n.mic.trim();});
        var micArray_  = _.allKeys(sortedArray);
        micArray = micArray_;
    });
}

/*
* Plots transfer function of mic/src
* @param  toSort - boolean
* @param  param1 - can be mic/src
* @param  param2 - can be mic/src
* @param  graphType_ - can be GT_MIC/GT_ID
* */
function plot(toSort, param1, param2, graphType_) {
    graphType = graphType_;
    Plotly.d3.csv('data.csv', function(err, rows){
        if(param2 != null) {
            if(graphType == GT_ID) {
                rows = _.filter(rows, function(num){ return num.id == param1 && num.mic == param2; });
            } else if(graphType == GT_MIC) {
                rows = _.filter(rows, function(num){ return num.mic == param1 && num.id == param2; });
            }

        } else {
            rows = _.filter(rows, function(num){return graphType == GT_ID? num.id == param1 : num.mic == param1; });
        }

        var start = new Date().getTime();
        if(toSort) {
            var maxValue = _.max(rows, function(n){ return Math.max(n.x); });
            var x_base = maxValue.x;

            var newSortedRow = new Array();

            // sorting the row by id
            var sortedArray = _.sortBy(rows, 'id');
            sortedArray = _.groupBy(sortedArray, function(n) {return n.id.trim();});
            var layout = {
                height: 768,
                width: 768,
                title: 'Transfer Function of Src No. '+ param1,
                scene: {
                    xaxis: {
                        title: "[Hz]"
                    },
                    yaxis: {
                        title: "Mic[No]"

                    },
                    zaxis: {
                        title: "[Amp]"
                    }
                }
            };

            if(graphType == GT_ID) {
                layout['title'] = 'Transfer Function of Src No. ' + param1;
                layout['scene']['yaxis']['title'] = "Mic[No]";
            } else if(graphType == GT_MIC) {
                layout['title'] = 'Transfer Function of Mic No. ' + param1;
                layout['scene']['yaxis']['title'] = "Src[No]";
            }

            for(var key in sortedArray) {
                newSortedRow[key] = _.sortBy(sortedArray[key], 'mic');
            }

            // collapse array as one
            newSortedRow = collapseArray(newSortedRow);
            rows = insertBlank(newSortedRow, x_base);
        } else {
            console.log("Not to sort");
        }

        function unpack(rows, key) {
            return rows.map(function(row) {
                return row[key];
            });
        }

        var x = unpack(rows , 'x');
        var id = unpack(rows , 'id');
        var mic = unpack(rows , 'mic');
        var z = unpack(rows , 'z');

        if(graphType == GT_ID) {
            Plotly.newPlot('graph', [{
                type: 'scatter3d',
                mode: 'lines',
                x: x,
                y: mic,
                z: z,
                opacity: 1,
                line: {
                    width: 1,
                    color: "#007acc"
                }
            }],layout);
        } else if(graphType == GT_MIC){
            Plotly.newPlot('graph', [{
                type: 'scatter3d',
                mode: 'lines',
                x: x,
                y: id,
                z: z,
                opacity: 1,
                line: {
                    width: 1,
                    color: "#007acc"
                }
            }],layout);
        }

        var end = new Date().getTime();
        var time = end - start;
        console.log('Execution time: ' + time + "ms");
    });
}

$( document ).ready(function() {
    $("#labelFilter").hide();
    $("#labelSelect").hide();
    $("#selectTF").hide();
    $("#filterTF").hide();

    getMicArray();
    getIdArray();

    $( "#sort" ).click(function() {
        $("input").remove();
        plot(true, 0, null, GT_MIC);
        $("#labelFilter").show();
        $("#labelSelect").show();
        $("#selectTF").show();
        $("#filterTF").show();
        var hiddenElement = document.createElement("input");
        hiddenElement.textContent = "0";
        hiddenElement.setAttribute("type", "hidden");
        $("body").append(hiddenElement);
        populateOptions(idArray);
    });

    $( "select#selectTF" ).change(function () {
        var value = $(this).val();
        $("input").remove();
        var hiddenValue = document.createElement("input");
        hiddenValue.textContent = value;
        hiddenValue.setAttribute("type", "hidden");
        $('body').append(hiddenValue);
        //debugger;
        if(graphType == GT_ID) {
            $("#labelFilter").text("Show only Src[no]: ");
            $("#labelSelect").text("Go to TF of Src[no]: ");
            if(value == "Please select") {
                plot(true, parseInt(hiddenValue), null, GT_MIC);
            } else {
                plot(true, parseInt(value), null, GT_MIC);
            }
            populateOptions(idArray);
        } else if(graphType == GT_MIC) {
            $("#labelFilter").text("Show only Mic[no]: ");
            $("#labelSelect").text("Go to TF of Mic[no]: ");
            if(value == "Please select") {
                plot(true, parseInt(hiddenValue), null, GT_ID);
            } else {
                plot(true, parseInt(value), null, GT_ID);
            }
            //debugger;
            populateOptions(micArray);
        }
    });

    $( "select#filterTF" ).change(function () {
        var hiddenValue = $("input").text();
        var value = $(this).val();
        if(graphType == GT_ID) {
            if(value == "Please select") {
                plot(true, parseInt(hiddenValue), null, GT_ID);
            } else {
                plot(true, parseInt(hiddenValue),  parseInt(value), GT_ID);
            }
        } else if(graphType == GT_MIC){
            if(value == "Please select") {
                plot(true, parseInt(hiddenValue), null, GT_MIC);
            } else {
                plot(true, parseInt(hiddenValue),  parseInt(value), GT_MIC);
            }
        }


    });

});